@@ -12,17 +12,22 @@
 
 RSA *wv_d2i_RSAPublicKey(RSA **a, const unsigned char **pp, long length)
 {
-    return d2i_RSAPublicKey(a, pp, length);
+    return d2i_RSAPublicKey(a, (void *)pp, length);
 }
 
 
 RSA *wv_d2i_RSAPrivateKey(RSA **a, const unsigned char **pp, long length)
 {
-    return d2i_RSAPrivateKey(a, pp, length);
+    return d2i_RSAPrivateKey(a, (void *)pp, length);
 }
 
 
+X509_REQ *wv_d2i_X509_REQ(X509_REQ **a, const unsigned char **pp, long length)
+{
+    return d2i_X509_REQ(a, (void *)pp, length);
+}
+
 X509 *wv_d2i_X509(X509 **a, unsigned char **pp, long length)
 {
-    return d2i_X509(a, pp, length);
+    return d2i_X509(a, (void *)pp, length);
 }
