@@ -5,15 +5,18 @@
  * X.509 certificate management classes.
  */ 
 #include "wvx509.h"
+#include "wvrsa.h"
 #include "wvsslhacks.h"
 #include "wvdiriter.h"
 #include "wvcrypto.h"
 #include "wvstringlist.h"
+#include "wvbase64.h"
 #include "strutils.h"
 
 #include <openssl/pem.h>
 #include <openssl/x509v3.h>
 #include <openssl/err.h>
+#include <openssl/ssl.h>
 #include <openssl/sha.h>
 #include <openssl/pkcs12.h>
 
@@ -64,32 +67,17 @@ WvX509Mgr::WvX509Mgr(X509 *_cert)
     rsa = NULL;
     if (cert)
     {
-	char buffer[1024];
-
-	X509_NAME_oneline(X509_get_subject_name(cert), buffer, sizeof(buffer));
-	buffer[sizeof(buffer)-1] = 0;
-	dname = buffer;
-	
-	EVP_PKEY *pk;
-	pk = X509_get_pubkey(cert);
-
-        if ((pk = X509_get_pubkey(cert)) != NULL)
-	{
- 	    size_t size;
-            unsigned char *keybuf, *iend;
-	    WvString tmppub;
-
-	    size = i2d_RSAPublicKey(pk->pkey.rsa, NULL);
-            iend = keybuf = new unsigned char[size];
-            i2d_RSAPublicKey(pk->pkey.rsa, &iend);
-
-            tmppub.setsize(size * 2 + 1);
-	    ::hexify(tmppub.edit(), keybuf, size);
-	    rsa = new WvRSAKey(tmppub, false);
-
-	    delete[] keybuf;
-	}
+	filldname();
+	rsa = fillRSAPubKey();
+	if (!rsa->isok())
+	    seterr("RSA Public Key Error: %s", rsa->errstr());
     }
+    else
+	;
+	// This isn't an error, or a mistake...
+	// so this is one case where 
+	// cert == NULL && rsa == NULL && errstr == NULL
+	// That the programmer should be doing something about.
 }
 
 
@@ -107,16 +95,16 @@ WvX509Mgr::WvX509Mgr(WvStringParm hexified_cert,
 	return;
     }
 
-    unhexify(hexified_cert);
+    if (!!hexified_cert)
+	unhexify(hexified_cert);
+    else
+    {
+	seterr("No Hexified Cert.. aborting!\n");
+	return;
+    }
 
     if (cert)
-    {
-	char buffer[1024];
-
-    	X509_NAME_oneline(X509_get_subject_name(cert), buffer, sizeof(buffer));
-    	buffer[sizeof(buffer)-1] = 0;
-    	dname = buffer;
-    }
+	filldname();
 }
 
 
@@ -139,8 +127,15 @@ WvX509Mgr::WvX509Mgr(WvStringParm _dname, int bits)
     wvssl_init();
     debug("Creating new certificate for %s\n", dname);
     cert = NULL;
-    rsa = new WvRSAKey(bits);
-    create_selfsigned();
+    rsa = NULL;
+    
+    if (!!dname)
+    {
+	rsa = new WvRSAKey(bits);
+	create_selfsigned();
+    }
+    else
+	seterr("Sorry, can't create an anonymous Certificate\n");
 }
 
 
@@ -148,11 +143,38 @@ WvX509Mgr::~WvX509Mgr()
 {
     if (rsa)
 	delete rsa;
-    X509_free(cert);
+
+    if (cert)
+	X509_free(cert);
+
     wvssl_free();
 }
 
 
+bool WvX509Mgr::bind_ssl(SSL_CTX *ctx)
+{
+    if (SSL_CTX_use_certificate(ctx, cert) <= 0)
+    {
+	return false;
+    }
+    debug("Certificate activated.\n");
+    
+    if (SSL_CTX_use_RSAPrivateKey(ctx, rsa->rsa) <= 0)
+    {
+	return false;
+    }
+    debug("RSA private key activated.\n");
+    return true;
+}
+
+
+const WvRSAKey &WvX509Mgr::get_rsa()
+{
+    assert(rsa);
+
+    return *rsa;
+}
+
 // The people who designed this garbage should be shot!
 // Support old versions of openssl...
 #ifndef NID_domainComponent
@@ -241,7 +263,7 @@ static WvString set_name_entry(X509_NAME *name, WvStringParm dn)
 }
 
 
-void WvX509Mgr::create_selfsigned()
+void WvX509Mgr::create_selfsigned(bool is_ca)
 {
     EVP_PKEY *pk = NULL;
     X509_NAME *name = NULL;
@@ -259,7 +281,7 @@ void WvX509Mgr::create_selfsigned()
     assert(rsa);
 
     // double check RSA key
-    if (RSA_check_key(rsa->rsa) == 1)
+    if (rsa->isok())
 	debug("RSA Key is fine.\n");
     else
     {
@@ -267,19 +289,19 @@ void WvX509Mgr::create_selfsigned()
 	return;
     }
 
-    if ((pk=EVP_PKEY_new()) == NULL)
+    if ((pk = EVP_PKEY_new()) == NULL)
     {
 	seterr("Error creating key handler for new certificate");
 	return;
     }
-    if ((cert=X509_new()) == NULL)
+    if ((cert = X509_new()) == NULL)
     {
 	seterr("Error creating new X509 object");
 	return;
     }
 
     // Assign RSA Key from WvRSAKey into stupid package that OpenSSL needs
-    if (!EVP_PKEY_assign_RSA(pk, rsa->rsa))
+    if (!EVP_PKEY_set1_RSA(pk, rsa->rsa))
     {
 	seterr("Error adding RSA keys to certificate");
 	return;
@@ -326,20 +348,25 @@ void WvX509Mgr::create_selfsigned()
     // Set the RFC2459-mandated keyUsage field to critical, and restrict
     // the usage of this cert to digital signature and key encipherment.
     ex = X509V3_EXT_conf_nid(NULL, NULL, NID_key_usage,
-	     "critical,digitalSignature,keyEncipherment,keyCertSign");
+	     "critical, digitalSignature, keyEncipherment, keyCertSign");
     X509_add_ext(cert, ex, -1);
     X509_EXTENSION_free(ex);
     
-#if 0
-    // This still causes Netscape to barf... 
-    ex = X509V3_EXT_conf_nid(NULL, NULL, NID_basic_constraints,
-			     "critical,CA:FALSE");
+    // This could cause Netscape to barf because if we set basicConstraints 
+    // to critical, we break RFC2459 compliance. Why they chose to enforce 
+    // that bit, and not the rest is beyond me... but oh well...
+    if (is_ca)
+	ex = X509V3_EXT_conf_nid(NULL, NULL, NID_basic_constraints,
+				 "critical, CA:TRUE");
+    else
+	ex = X509V3_EXT_conf_nid(NULL, NULL, NID_basic_constraints,
+				 "CA:FALSE");
+	
     X509_add_ext(cert, ex, -1);
     X509_EXTENSION_free(ex);
-#endif
     
     ex = X509V3_EXT_conf_nid(NULL, NULL, NID_ext_key_usage,
-	     "TLS Web Server Authentication,TLS Web Client Authentication");
+	     "TLS Web Server Authentication, TLS Web Client Authentication");
     X509_add_ext(cert, ex, -1);
     X509_EXTENSION_free(ex);
 
@@ -351,11 +378,33 @@ void WvX509Mgr::create_selfsigned()
 	EVP_PKEY_free(pk);
 	return;
     }
-
+    
+    EVP_PKEY_free(pk);
+    
     debug("Certificate for %s created\n", dname);
 }
 
 
+void WvX509Mgr::filldname()
+{
+    assert(cert);
+    
+    char *name = X509_NAME_oneline(X509_get_subject_name(cert), 0, 0);
+    dname = name;
+    OPENSSL_free(name);
+}
+
+
+// FIXME: This is EVIL!!!
+WvRSAKey *WvX509Mgr::fillRSAPubKey()
+{
+    EVP_PKEY *pkcert = X509_get_pubkey(cert);
+    RSA *certrsa = EVP_PKEY_get1_RSA(pkcert);
+    EVP_PKEY_free(pkcert);
+    return new WvRSAKey(certrsa, false); 
+}
+
+
 static FILE *file_hack_start()
 {
     return tmpfile();
@@ -379,9 +428,9 @@ static WvString file_hack_end(FILE *f)
 
 WvString WvX509Mgr::certreq()
 {
-    EVP_PKEY *pk;
+    EVP_PKEY *pk = NULL;
     X509_NAME *name = NULL;
-    X509_REQ *certreq;
+    X509_REQ *certreq = NULL;
 
     FILE *stupid = file_hack_start();
     
@@ -389,7 +438,7 @@ WvString WvX509Mgr::certreq()
     assert(dname);
 
     // double check RSA key
-    if (RSA_check_key(rsa->rsa) == 1)
+    if (rsa->isok())
 	debug("RSA Key is fine.\n");
     else
     {
@@ -410,8 +459,7 @@ WvString WvX509Mgr::certreq()
         return WvString::null;
     }
 
-    WvRSAKey rsa2(*rsa);
-    if (!EVP_PKEY_assign_RSA(pk, rsa2.rsa))
+    if (!EVP_PKEY_set1_RSA(pk, rsa->rsa))
     {
         seterr("Error adding RSA keys to certificate");
 	X509_REQ_free(certreq);
@@ -428,9 +476,12 @@ WvString WvX509Mgr::certreq()
     debug("Creating Certificate request for %s\n", dname);
     set_name_entry(name, dname);
     X509_REQ_set_subject_name(certreq, name);
-    debug("SubjectDN: %s\n",
-	      X509_NAME_oneline(X509_REQ_get_subject_name(certreq), 0, 0));
+    char *sub_name = X509_NAME_oneline(X509_REQ_get_subject_name(certreq), 
+				       0, 0);
 
+    debug("SubjectDN: %s\n", sub_name);
+    OPENSSL_free(sub_name);
+    
     if (!X509_REQ_sign(certreq, pk, EVP_sha1()))
     {
 	seterr("Could not self sign the request");
@@ -452,10 +503,6 @@ WvString WvX509Mgr::certreq()
 	debug("Self Signed Certificate Request verifies OK!\n");
     }
 
-    // don't let rsa2 free its own RSA key... the certificate will do that
-    // for us.
-    rsa2.rsa = NULL;
-
     // Horribly involuted hack to get around the fact that the
     // OpenSSL people are too braindead to have a PEM_write function
     // that returns a char *
@@ -465,7 +512,120 @@ WvString WvX509Mgr::certreq()
   
     return file_hack_end(stupid);
 }
- 
+
+
+WvString WvX509Mgr::signcert(WvStringParm pkcs10req)
+{
+    assert(rsa);
+    
+    WvString pkcs10(pkcs10req);
+    
+    char *begin = strstr(pkcs10.edit(), "\nMII") + 1;
+    if (!begin)
+    {
+	debug("This doesn't look like PEM Encoded information...\n");
+	return WvString::null;
+    }
+    char *end = strstr(begin, "=\n---") + 1 ;
+    if (!end)
+    {
+	debug("Is this a complete certificate request?\n");
+	return WvString::null;
+    }
+    *end = '\0';
+    WvString body(begin); // just the PKCS#10 request, 
+                          // without the ---BEGIN and ---END
+    
+    WvDynBuf reqbuf;
+    WvBase64Decoder dec;
+    dec.flushstrbuf(body, reqbuf, true);
+    
+    // FIXME: Duplicates code from cert_selfsign
+    size_t reqlen = reqbuf.used();
+    const unsigned char *req = reqbuf.get(reqlen);
+    X509_REQ *certreq = wv_d2i_X509_REQ(NULL, &req, reqlen);
+    if (certreq)
+    {
+	X509 *newcert = X509_new();
+
+	// Set the subject name of the new certificate to be 
+	// exactly the subject name of the request.
+	X509_set_subject_name(newcert, X509_REQ_get_subject_name(certreq));
+
+	// Completely broken in my mind - this sets the version
+	// string to '3'  (I guess version starts at 0)
+	X509_set_version(newcert, 0x2);
+	
+	// Set the Serial Number for the certificate
+	srand(time(NULL));
+	int serial = rand();
+	ASN1_INTEGER_set(X509_get_serialNumber(newcert), serial);
+	
+	// Set the NotBefore time to now.
+	X509_gmtime_adj(X509_get_notBefore(newcert), 0);
+	
+	// Now + 10 years... should be shorter, but since we don't currently
+	// have a set of routines to refresh the certificates, make it
+	// REALLY long.
+	X509_gmtime_adj(X509_get_notAfter(newcert), (long)60*60*24*3650);
+
+	// The public key of the new cert should be the same as that from 
+	// the request.
+	EVP_PKEY *pk = X509_REQ_get_pubkey(certreq);
+	X509_set_pubkey(newcert, pk);
+	EVP_PKEY_free(pk);
+	
+	// The Issuer name is the subject name of the current cert
+	X509_set_issuer_name(newcert, X509_get_subject_name(cert));
+	
+	X509_EXTENSION *ex = NULL;
+	// Set the RFC2459-mandated keyUsage field to critical, and restrict
+	// the usage of this cert to digital signature and key encipherment.
+	ex = X509V3_EXT_conf_nid(NULL, NULL, NID_key_usage,
+				 "critical, digitalSignature, keyEncipherment");
+	X509_add_ext(newcert, ex, -1);
+	X509_EXTENSION_free(ex);
+    
+	// This could cause Netscape to barf because if we set basicConstraints 
+	// to critical, we break RFC2459 compliance. Why they chose to enforce 
+	// that bit, and not the rest is beyond me... but oh well...
+	ex = X509V3_EXT_conf_nid(NULL, NULL, NID_basic_constraints,
+				 "CA:FALSE");
+	
+	X509_add_ext(newcert, ex, -1);
+	X509_EXTENSION_free(ex);
+
+	ex = X509V3_EXT_conf_nid(NULL, NULL, NID_ext_key_usage,
+				 "critical, TLS Web Client Authentication");
+	X509_add_ext(newcert, ex, -1);
+	X509_EXTENSION_free(ex);
+
+	// Ok, now sign the new cert with the current RSA key
+	EVP_PKEY *certkey = EVP_PKEY_new();
+	bool cakeyok = EVP_PKEY_set1_RSA(certkey, rsa->rsa);
+	if (newcert && cakeyok)
+	    X509_sign(newcert, certkey, EVP_sha1());
+	else
+	{
+	    debug("No keys??\n");
+	    return WvString::null;
+	}
+	
+	EVP_PKEY_free(certkey);
+	
+        // Note - takes ownership of newcert, and will delete it for us.
+	WvX509Mgr nmgr(newcert);
+	
+	X509_REQ_free(certreq);
+	return WvString(nmgr.encode(CertPEM));
+    }
+    else
+    {
+	debug("Can't decode Certificate Request\n");
+	return WvString::null;
+    }
+}
+
 
 bool WvX509Mgr::test()
 {
@@ -481,9 +641,7 @@ bool WvX509Mgr::test()
     
     if (rsa && pk)
     {
-	WvRSAKey tmpkey(*rsa);
-	
-	if (!EVP_PKEY_assign_RSA(pk, tmpkey.rsa))
+	if (!EVP_PKEY_set1_RSA(pk, rsa->rsa))
     	{
             seterr("Error setting RSA keys");
 	    bad = true;
@@ -493,12 +651,23 @@ bool WvX509Mgr::test()
 	    int verify_return = X509_verify(cert, pk);
 	    if (verify_return != 1) // only '1' means okay
 	    {
-		seterr("Certificate test failed: %s\n", wvssl_errstr());
-		bad = true;
+		// However let's double check:
+		WvString rsapub = encode(RsaPubPEM);
+		WvRSAKey *temprsa = fillRSAPubKey();
+		WvString certpub = temprsa->getpem(false);
+		delete temprsa;
+		// debug("rsapub:\n%s\n", rsapub);
+		// debug("certpub:\n%s\n", certpub);
+		if (certpub == rsapub)
+		    ; // do nothing, since OpenSSL is lying
+		else
+		{
+		    // I guess that it really did fail.
+		    seterr("Certificate test failed: %s\n", wvssl_errstr());
+		    bad = true;
+		}
 	    }
 	}
-	
-	tmpkey.rsa = NULL;
     }
     else
     {
@@ -515,6 +684,12 @@ bool WvX509Mgr::test()
 
 void WvX509Mgr::unhexify(WvStringParm encodedcert)
 {
+    if (!encodedcert)
+    {
+	seterr("X.509 certificate can't be decoded from nothing!\n");
+	return;
+    }
+    
     int hexbytes = strlen(encodedcert.cstr());
     int bufsize = hexbytes/2;
     unsigned char *certbuf = new unsigned char[bufsize];
@@ -538,7 +713,7 @@ void WvX509Mgr::unhexify(WvStringParm encodedcert)
     if (!cert)
 	seterr("X.509 certificate decode failed!");
     
-    delete[] certbuf;
+    deletev certbuf;
 }
 
 
@@ -555,26 +730,27 @@ WvString WvX509Mgr::hexify()
     enccert.setsize(size * 2 +1);
     ::hexify(enccert.edit(), keybuf, size);
 
-    delete[] keybuf;
+    deletev keybuf;
     return enccert;
 }
 
 
-bool WvX509Mgr::validate()
+bool WvX509Mgr::validate(WvX509Mgr *cacert, X509_CRL *crl)
 {
+    bool retval = true;
+    
     if (cert != NULL)
     {
-	debug("Peer Certificate:\n");
-	debug("Issuer: %s\n",
-	      X509_NAME_oneline(X509_get_issuer_name(cert),0,0));
-
 	// Check and make sure that the certificate is still valid
 	if (X509_cmp_current_time(X509_get_notAfter(cert)) == -1)
 	{
-	    seterr("Peer certificate has expired!");
-	    return false;
+	    seterr("Certificate has expired!");
+	    retval = false;
 	}
 	
+	if (cacert)
+	    retval &= signedbyCA(cacert);
+
 	// Kind of a placeholder thing right now...
 	// Later on, do CRL, and certificate validity checks here..
         // Actually, break these out in signedbyvalidCA(), and isinCRL()
@@ -584,7 +760,7 @@ bool WvX509Mgr::validate()
     else
 	debug("Peer doesn't have a certificate.\n");
     
-    return true;
+    return retval;
 }
 
 
@@ -602,7 +778,7 @@ bool WvX509Mgr::signedbyCAinfile(WvStringParm certfile)
 	return false;
     }
 
-    lookup=X509_STORE_add_lookup(cert_ctx, X509_LOOKUP_file());
+    lookup = X509_STORE_add_lookup(cert_ctx, X509_LOOKUP_file());
     if (lookup == NULL)
     {
 	seterr("Can't add lookup method...\n");
@@ -637,16 +813,19 @@ bool WvX509Mgr::signedbyCAindir(WvStringParm certdir)
 }
 
 
-bool WvX509Mgr::isinCRL()
+bool WvX509Mgr::signedbyCA(WvX509Mgr *cacert)
 {
-    return false;
+    int ret = X509_check_issued(cacert->cert, cert);
+    if (ret == X509_V_OK)
+	return true;
+    else
+	return false;
 }
 
 
 WvString WvX509Mgr::encode(const DumpMode mode)
 {
-    FILE *stupid;
-    const EVP_CIPHER *enc;
+    FILE *stupid = NULL;
     WvString nil;
     
     stupid = file_hack_start();
@@ -662,11 +841,15 @@ WvString WvX509Mgr::encode(const DumpMode mode)
 	    
 	case RsaPEM:
 	    debug("Dumping RSA keypair.\n");
-	    enc = EVP_get_cipherbyname("rsa");
-	    PEM_write_RSAPrivateKey(stupid, rsa->rsa, enc,
-				    NULL, 0, NULL, NULL);
+	    fclose(stupid);
+	    return rsa->getpem(true);
 	    break;
 	    
+	case RsaPubPEM:
+	    debug("Dumping RSA Public Key!\n");
+	    fclose(stupid);
+	    return rsa->getpem(false);
+	    break;
 	case RsaRaw:
 	    debug("Dumping raw RSA keypair.\n");
 	    RSA_print_fp(stupid, rsa->rsa, 0);
@@ -686,13 +869,18 @@ WvString WvX509Mgr::encode(const DumpMode mode)
     }
 }
 
-void WvX509Mgr::decode(DumpMode mode, WvStringParm pemEncoded)
+void WvX509Mgr::decode(const DumpMode mode, WvStringParm pemEncoded)
 {
+    if (!pemEncoded)
+    {
+	debug(WvLog::Error, "Not decoding an empty string. - Sorry!\n");
+	return;
+    }
+    
     // Let the fun begin... ;)
-    FILE *stupid;
-    const EVP_CIPHER *enc;
+    FILE *stupid = NULL;
     WvString outstring = pemEncoded;
-                
+    
     stupid = file_hack_start();
 
     if (stupid)
@@ -717,15 +905,34 @@ void WvX509Mgr::decode(DumpMode mode, WvStringParm pemEncoded)
 		cert = NULL;
 	    }
 	    cert = PEM_read_X509(stupid, NULL, NULL, NULL);
+	    if (cert)
+	    {
+		filldname();
+		if (!rsa)
+		    rsa = fillRSAPubKey();
+	    }
+	    else
+		seterr("Certificate failed to import!");
 	    break;
 	    
 	case RsaPEM:
 	    debug("Importing RSA keypair.\n");
 	    debug("Make sure that you load or generate a new Certificate!\n");
-	    enc = EVP_get_cipherbyname("rsa");
-	    rsa->rsa = PEM_read_RSAPrivateKey(stupid, NULL, NULL, NULL);
+	    if (rsa) delete rsa;
+	    rsa = new WvRSAKey(PEM_read_RSAPrivateKey(stupid, NULL, NULL, NULL), 
+			       true);
+	    if (!rsa->isok())
+		seterr("RSA Key failed to import\n");
+	    break;
+	case RsaPubPEM:
+	    debug("Importing RSA Public Key.\n");
+	    debug("Are you REALLY sure that you want to do this?\n");
+	    if (rsa) delete rsa;
+	    rsa = new WvRSAKey(PEM_read_RSAPublicKey(stupid, NULL, NULL, NULL), 
+			       true);
+	    if (!rsa->isok())
+		seterr("RSA Public Key failed to import\n");
 	    break;
-	    
 	case RsaRaw:
 	    debug("Importing raw RSA keypair not supported.\n");
 	    break;
@@ -742,34 +949,53 @@ void WvX509Mgr::decode(DumpMode mode, WvStringParm pemEncoded)
     }
 }
 
+namespace {
+class AutoClose {
+public:
+    AutoClose(FILE *fp): fp(fp) { }
+    ~AutoClose()
+    {
+        if (fp)
+            fclose(fp);
+    }
+    
+    operator FILE *() const
+    {
+	return fp;
+    }
+    
+private:
+    FILE *fp;
+};
+} // anomymous namespace...
+
 void WvX509Mgr::write_p12(WvStringParm filename)
 {
     debug("Dumping RSA Key and X509 Cert to PKCS12 structure.\n");
 
-    FILE *fp = fopen(filename, "w");
-    
+    AutoClose fp = fopen(filename, "w");
+
     if (!fp)
     {
-	seterr("Unable to create: %s\n", filename);
-	return;
+        seterr("Unable to create: %s\n", filename);
+        return;
     }
-    
+
     if (!!pkcs12pass)
     {
-	EVP_PKEY *pk = EVP_PKEY_new();
-	if (!pk)
-	{
-	    seterr("Unable to create PKEY object.\n");
-	    return;
-	}
-	
 	if (rsa && cert)
 	{
-	    WvRSAKey tmpkey(*rsa);
-	    
-	    if (!EVP_PKEY_assign_RSA(pk, tmpkey.rsa))
+	    EVP_PKEY *pk = EVP_PKEY_new();
+	    if (!pk)
+	    {
+		seterr("Unable to create PKEY object.\n");
+		return;
+	    }
+
+	    if (!EVP_PKEY_set1_RSA(pk, rsa->rsa))
 	    {
 		seterr("Error setting RSA keys.\n");
+		EVP_PKEY_free(pk);
 		return;
 	    }
 	    else
@@ -781,10 +1007,12 @@ void WvX509Mgr::write_p12(WvStringParm filename)
 		    debug("Write the PKCS12 object out...\n");
 		    i2d_PKCS12_fp(fp, pkg);
 		    PKCS12_free(pkg);
+		    EVP_PKEY_free(pk);
 		}
 		else
 		{
 		    seterr("Unable to create PKCS12 object.\n");
+		    EVP_PKEY_free(pk);
 		    return;
 		}
 	    }
@@ -797,23 +1025,23 @@ void WvX509Mgr::write_p12(WvStringParm filename)
     }
     else
     {
-	seterr("No Password specified for PKCS12 dump\n");
-	return;
+        seterr("No Password specified for PKCS12 dump\n");
+        return; 
     }
 }
 
 void WvX509Mgr::read_p12(WvStringParm filename)
 {
     debug("Reading Certificate and Private Key from PKCS12 file: %s\n", filename);
-    
-    FILE *fp = fopen(filename, "r");
-    
+
+    AutoClose fp = fopen(filename, "r");
+
     if (!fp)
     {
-	seterr("Unable to read from: %s\n", filename);
-	return;
+        seterr("Unable to read from: %s\n", filename);
+        return;
     }
-    
+
     if (!!pkcs12pass)
     {
 	PKCS12 *pkg = d2i_PKCS12_fp(fp, NULL);
@@ -833,7 +1061,7 @@ void WvX509Mgr::read_p12(WvStringParm filename)
 	    // Now, cert should be OK, let's try and set up the RSA stuff
 	    // since we've essentially got a PKEY, and not a WvRSAKey
 	    // We need to create a new WvRSAKey from the PKEY...
-	    rsa = new WvRSAKey(pk->pkey.rsa, true);
+	    rsa = new WvRSAKey(EVP_PKEY_get1_RSA(pk), true);
 	    
 	    // Now that we have both, check to make sure that they match
 	    if (!rsa || !cert || test())
@@ -841,8 +1069,7 @@ void WvX509Mgr::read_p12(WvStringParm filename)
 		seterr("Could not fill in RSA and Cert with matching values.\n");
 		return;
 	    }
-	    // We don't free pk here because the WvRSAKey object now 
-	    // owns the key value and should delete it in the constructor.
+	    EVP_PKEY_free(pk);
 	}
 	else
 	{
@@ -852,79 +1079,236 @@ void WvX509Mgr::read_p12(WvStringParm filename)
     }
     else
     {
-	seterr("No Password specified for PKCS12 file - aborting!\n");
-	return;
+        seterr("No Password specified for PKCS12 file - aborting!\n");
+        return;
     }
 }
 
 
 WvString WvX509Mgr::get_issuer()
 { 
-    return WvString(X509_NAME_oneline(X509_get_issuer_name(cert),0,0)); 
+    if (cert)
+    {
+	char *name = X509_NAME_oneline(X509_get_issuer_name(cert),0,0);
+        WvString retval(name);
+        OPENSSL_free(name);
+	return retval;
+    }
+    else
+	return WvString::null;
 }
 
 
 WvString WvX509Mgr::get_subject()
 {
-    return WvString(X509_NAME_oneline(X509_get_subject_name(cert),0,0));
+    if (cert)
+    {
+	char *name = X509_NAME_oneline(X509_get_subject_name(cert), 0, 0);
+	WvString retval(name);
+	OPENSSL_free(name);
+	return retval;
+    }
+    else
+	return WvString::null;
+}
+
+WvString WvX509Mgr::get_serial()
+{
+    if (cert)
+    {
+	return WvString(ASN1_INTEGER_get(X509_get_serialNumber(cert)));
+    }
+    else
+	return WvString::null;
 }
 
 
 WvString WvX509Mgr::get_crl_dp()
 {
-    WvDynBuf *buf = get_extension(NID_crl_distribution_points);
-    if (buf)
-    {
-	WvString retval(buf->getstr());
-	delete buf;
-	return retval;
-    }
-    else
-	return WvString::null;
+    return get_extension(NID_crl_distribution_points);
 }
 
 
 WvString WvX509Mgr::get_cp_oid()
 {
-    WvDynBuf *buf = get_extension(NID_certificate_policies);
-    if (buf)
-    {
-	WvString retval(buf->getstr());
-	delete buf;
-	return retval;
-    }
-    else
-	return WvString::null;
+    return get_extension(NID_certificate_policies);
 }
 
 
 WvString WvX509Mgr::get_altsubject()
 {
-    WvDynBuf *buf = get_extension(NID_subject_alt_name);
-    if (buf)
-    {
-	WvString retval(buf->getstr());
-	delete buf;
-	return retval;
-    }
-    else
-	return WvString::null;
-
+    return get_extension(NID_subject_alt_name);
 }
 
-WvDynBuf *WvX509Mgr::get_extension(int nid)
+WvString WvX509Mgr::get_extension(int nid)
 {
-    int index = X509_get_ext_by_NID(cert, nid, -1);
-    if (index >= 0)
+    WvString retval = WvString::null;
+    
+    if (cert)
     {
-	X509_EXTENSION *ext = X509_get_ext(cert, index);
-	if (ext)
+	int index = X509_get_ext_by_NID(cert, nid, -1);
+	if (index >= 0)
 	{
-	    WvDynBuf *buf = new WvDynBuf();
-	    buf->put(ext->value->data, ext->value->length);
-	    return buf;
+	    X509_EXTENSION *ext = X509_get_ext(cert, index);
+	    if (ext)
+	    {
+		X509V3_EXT_METHOD *method = X509V3_EXT_get(ext);
+		if (!method)
+		{
+		    WvDynBuf buf;
+		    buf.put(ext->value->data, ext->value->length);
+		    retval = buf.getstr();
+		}
+		else
+		{
+		    void *ext_data = NULL;
+		    if (method->it) 
+			ext_data = ASN1_item_d2i(NULL, &ext->value->data, 
+						ext->value->length, 
+						ASN1_ITEM_ptr(method->it));
+		    else
+			ext_data = method->d2i(NULL, &ext->value->data, 
+					      ext->value->length);
+		    
+		    if (method->i2s)
+		    {
+			retval = method->i2s(method, ext_data);
+			if (method->it)
+			    ASN1_item_free((ASN1_VALUE *)ext_data, ASN1_ITEM_ptr(method->it));
+			else
+			    method->ext_free(ext_data);
+		    }
+		    else if (method->i2v)
+			retval = "Stack type!";
+		    else if (method->i2r)
+			retval = "Guess what - it's raw..!";
+		}
+	    }
 	}
     }
-    return NULL;
+
+    if (!!retval)
+	return retval;
+    else
+	return WvString::null;
 }
 
+
+bool WvX509Mgr::isok() const
+{
+    return cert && rsa && WvError::isok();
+}
+
+
+WvString WvX509Mgr::errstr() const
+{
+    if (WvError::geterr() == 0)
+    {
+        // only use a custom string if there's not an error set
+        if (!cert && !rsa)
+            return "No certificate or RSA key assigned";
+        else if (!cert)
+            return "No certificate assigned";
+        else if (!rsa)
+            return "No RSA key assigned";
+    }
+    return WvError::errstr();
+}
+
+
+int WvX509Mgr::geterr() const
+{
+    int ret = WvError::geterr();
+    if (ret == 0 && (!cert || !rsa))
+    {
+        // unless there's a regular error set, we'll be returning a custom
+        // string: see errstr()
+        ret = -1;
+    }
+    return ret;
+}
+
+WvString WvX509Mgr::sign(WvStringParm data)
+{
+    WvDynBuf buf;
+    buf.putstr(data);
+    return sign(buf);
+}
+
+WvString WvX509Mgr::sign(WvBuf &data)
+{
+    assert(rsa);
+
+    EVP_MD_CTX sig_ctx;
+    unsigned char sig_buf[4096];
+    
+    EVP_PKEY *pk = EVP_PKEY_new();
+    if (!pk)
+    {
+	seterr("Unable to create PKEY object.\n");
+	return WvString::null;
+    }
+    
+    if (!EVP_PKEY_set1_RSA(pk, rsa->rsa))
+    {
+	seterr("Error setting RSA keys.\n");
+	EVP_PKEY_free(pk);
+	return WvString::null;
+    }
+    
+    EVP_SignInit(&sig_ctx, EVP_sha1());
+    EVP_SignUpdate(&sig_ctx, data.peek(0, data.used()), data.used());
+    unsigned int sig_len = sizeof(sig_buf);
+    int sig_err = EVP_SignFinal(&sig_ctx, sig_buf, 
+				&sig_len, pk);
+    if (sig_err != 1)
+    {
+	seterr("Error while signing!\n");
+	EVP_PKEY_free(pk);
+	return WvString::null;
+    }
+
+    EVP_PKEY_free(pk);
+    EVP_MD_CTX_cleanup(&sig_ctx); // this isn't my fault ://
+    WvDynBuf buf;
+    buf.put(sig_buf, sig_len);
+    debug("Signature size: %s\n", buf.used());
+    return WvBase64Encoder().strflushbuf(buf, true);
+}
+
+bool WvX509Mgr::verify(WvStringParm original, WvStringParm signature)
+{
+    WvDynBuf buf;
+    buf.putstr(original);
+    return verify(buf, signature);
+}
+
+bool WvX509Mgr::verify(WvBuf &original, WvStringParm signature)
+{
+    
+    unsigned char sig_buf[4096];
+    size_t sig_size = sizeof(sig_buf);
+    WvBase64Decoder().flushstrmem(signature, sig_buf, &sig_size, true);
+    
+    EVP_PKEY *pk = X509_get_pubkey(cert);
+    if (!pk) 
+    {
+        seterr("Couldn't allocate PKEY for verify()\n");
+        return false;
+    }
+    
+    /* Verify the signature */
+    EVP_MD_CTX sig_ctx;
+    EVP_VerifyInit(&sig_ctx, EVP_sha1());
+    EVP_VerifyUpdate(&sig_ctx, original.peek(0, original.used()), original.used());
+    int sig_err = EVP_VerifyFinal(&sig_ctx, sig_buf, sig_size, pk);
+    EVP_PKEY_free(pk);
+    EVP_MD_CTX_cleanup(&sig_ctx); // Again, not my fault... 
+    if (sig_err != 1) 
+    {
+        debug("Verify failed!\n");
+        return false;
+    }
+    else
+	return true;
+}
