@@ -7,7 +7,13 @@
 #ifndef __UNICONF_H
 #define __UNICONF_H
 
-#ifdef	__cplusplus
+
+/**
+ * C++ Interface
+ */
+
+
+#ifdef  __cplusplus
 
 #include "uniconfkey.h"
 #include "uniconfgen.h"
@@ -27,6 +33,11 @@ class UniConfRoot;
  */
 typedef WvCallback<void, const UniConf &, const UniConfKey &> UniConfCallback;
 
+#ifdef SWIG_NO_OVERLOAD
+// FIXME: This directive doesn't work.  Why not?
+%ignore UniConf::u(const UniConfKey &key);
+#endif
+
 /**
  * UniConf instances function as handles to subtrees of a UniConf
  * tree and expose a high-level interface for clients.
@@ -41,7 +52,7 @@ typedef WvCallback<void, const UniConf &, const UniConfKey &> UniConfCallback;
  * const to guard against accidentally assigning to a temporary by
  * an expression such as cfg["foo"] = cfg["bar"].
  * Instead this must be written as
- *     cfg["foo"].set(cfg["bar"].get())
+ *     cfg["foo"].setme(cfg["bar"].getme())
  * which is slightly
  * less elegant but avoids many subtle mistakes.  Also for this
  * reason, unusual cast operators, assignment operators,
@@ -62,23 +73,17 @@ protected:
      * You can't create non-NULL UniConf objects yourself - ask UniConfRoot
      * or another UniConf object to make one for you.
      */
-    UniConf(UniConfRoot *root, const UniConfKey &fullkey = UniConfKey::EMPTY)
-        : xroot(root), xfullkey(fullkey) { }
+    UniConf(UniConfRoot *root, const UniConfKey &fullkey = UniConfKey::EMPTY);
     
 public:
     /** Creates a NULL UniConf handle, useful for reporting errors. */
-    UniConf() 
-        : xroot(NULL), xfullkey(UniConfKey::EMPTY) { }
+    UniConf();
     
-    /**
-     * Copies a UniConf handle.
-     * "other" is the handle to copy
-     */
-    UniConf(const UniConf &other)
-        : xroot(other.xroot), xfullkey(other.xfullkey) { }
+    /** Copies a UniConf handle. */
+    UniConf(const UniConf &other);
     
     /** Destroys the UniConf handle. */
-    virtual ~UniConf() { }
+    virtual ~UniConf();
 
     
     /***** Handle Manipulation API *****/
@@ -108,9 +113,16 @@ public:
     
     /** Returns the full path of this node, starting at the given key.
      * Assumes that k is an ancestor of fullkey(). */
+#ifdef SWIG_NO_OVERLOAD
+    %name(fullkey_from_key)
+#endif
     UniConfKey fullkey(const UniConfKey &k) const;
     
     /** Returns the full path of this node, starting at the given handle. */
+#ifdef SWIG_NO_OVERLOAD
+    %name(fullkey_from_handle)
+#endif
+
     UniConfKey fullkey(const UniConf &cfg) const
         { return fullkey(cfg.fullkey()); }
 
@@ -118,6 +130,7 @@ public:
     UniConfKey key() const
         { return xfullkey.last(); }
 
+#ifndef SWIG
     /**
      * Returns a handle for a subtree below this key. 'key' is the path
      * of the subtree to be appended to the full path of this handle to
@@ -125,6 +138,17 @@ public:
      */
     const UniConf operator[] (const UniConfKey &key) const
         { return UniConf(xroot, UniConfKey(xfullkey, key)); }
+#endif  // SWIG
+
+    /**
+     * Return a subtree handle (see operator[]).  Mainly to support bindings
+     * for languages that can't handle methods named [].
+     */
+#ifdef SWIG_NO_OVERLOAD
+    %name(u_should_be_ignored)
+#endif
+    const UniConf u(const UniConfKey &key) const
+        { return (*this)[key]; }
 
     /** Reassigns the target of this handle to match a different one. */
     UniConf &operator= (const UniConf &other)
@@ -137,11 +161,40 @@ public:
     
     /***** Key Retrieval API *****/
     
+    /** See UniConfGen::prefetch(). */
+    void prefetch(bool recursive) const;
+    
     /**
      * Fetches the string value for this key from the registry.  If the
      * key is not found, returns 'defvalue' instead.
      */
-    WvString get(WvStringParm defvalue = WvString::null) const;
+    WvString getme(WvStringParm defvalue = WvString::null) const;
+
+    /** A different way to say cfg.getme(): use *cfg instead. */
+    WvString operator* () const
+        { return getme(); }
+
+    /** A different way to say cfg.getme().num(): use cfg->num() instead. */
+    WvStringStar operator -> () const
+        { return getme(); }
+    
+    /** A different way to say cfg[x].getme(y). */
+    WvString xget(WvStringParm key,
+		  WvStringParm defvalue = WvString::null) const
+        { return (*this)[key].getme(defvalue); }
+
+    /**
+     * Fetches the integer value for this key from the registry.  If the
+     * key is not found, returns 'defvalue' instead.  (This is also used to
+     * fetch booleans - 'true', 'yes', 'on' and 'enabled' are recognized as
+     * 1, 'false', 'no', 'off' and 'disabled' as 0.  Note that a nonexistant
+     * key is false by default.)
+     */
+    int getmeint(int defvalue = 0) const;
+
+    /** A different way to say cfg[x].getmeint(y). */
+    int xgetint(WvStringParm key, int defvalue = 0) const
+        { return (*this)[key].getmeint(defvalue); }
 
     /**
      * Without fetching its value, returns true if this key exists.
@@ -151,15 +204,6 @@ public:
      */
     bool exists() const;
 
-    /**
-     * Fetches the integer value for this key from the registry.  If the
-     * key is not found, returns 'defvalue' instead.  (This is also used to
-     * fetch booleans - 'true', 'yes', 'on' and 'enabled' are recognized as
-     * 1, 'false', 'no', 'off' and 'disabled' as 0.  Note that a nonexistant
-     * key is false by default.)
-     */
-    int getint(int defvalue = 0) const;
-
 
     /***** Key Storage API *****/
 
@@ -168,20 +212,28 @@ public:
      * is WvString::null, deletes the key and all of its children.
      * Returns true on success.
      */
-    void set(WvStringParm value) const;
+    void setme(WvStringParm value) const;
 
     /**
      * Stores a string value for this key into the registry.
      * Returns true on success.
      */
-    void set(WVSTRING_FORMAT_DECL) const
-        { return set(WvString(WVSTRING_FORMAT_CALL)); }
+    void setme(WVSTRING_FORMAT_DECL) const
+        { return setme(WvString(WVSTRING_FORMAT_CALL)); }
+
+    /** A different way to say cfg[x].setme(y). */
+    void xset(WvStringParm key, WvStringParm value) const
+        { (*this)[key].setme(value); }
 
     /**
      * Stores an integer value for this key into the registry.
      * Returns true on success.
      */
-    void setint(int value) const;
+    void setmeint(int value) const;
+
+    /** A different way to say cfg[x].setme(y). */
+    void xsetint(WvStringParm key, int value) const
+        { (*this)[key].setmeint(value); }
 
 
     /***** Key Handling API *****/
@@ -199,14 +251,14 @@ public:
      * Unlike unix mv(), this is *not* currently atomic.  It's more like
      * cp-then-rm.
      */
-    void move(const UniConf &dst);
+    void move(const UniConf &dst) const;
 
     /**
      * Removes this key and all of its children from the registry.
      * Returns true on success.
      */
     void remove() const
-        { set(WvString::null); }
+        { setme(WvString::null); }
 
     /**
      * Equivalent to "cp -r" in a standard unix filesystem. This
@@ -246,7 +298,7 @@ public:
      *
      * Returns the mounted generator, or NULL on failure.
      */
-    UniConfGen *mount(WvStringParm moniker, bool refresh = true) const;
+    IUniConfGen *mount(WvStringParm moniker, bool refresh = true) const;
     
     /**
      * Mounts a generator at this key.
@@ -258,10 +310,10 @@ public:
      * 
      * Returns the mounted generator, or NULL on failure.
      */
-    UniConfGen *mountgen(UniConfGen *gen, bool refresh = true) const;
+    IUniConfGen *mountgen(IUniConfGen *gen, bool refresh = true) const;
     
     /** Unmounts the generator providing this key and destroys it. */
-    void unmount(UniConfGen *gen, bool commit) const;
+    void unmount(IUniConfGen *gen, bool commit) const;
     
     /** Determines if any generators are mounted at this key. */
     bool ismountpoint() const;
@@ -276,7 +328,7 @@ public:
      * If non-NULL, 'mountpoint' is set to the actual key where the generator
      * is mounted.
      */
-    UniConfGen *whichmount(UniConfKey *mountpoint = NULL) const;
+    IUniConfGen *whichmount(UniConfKey *mountpoint = NULL) const;
 
     
     /***** Notification API *****/
@@ -372,12 +424,16 @@ public:
     class SortedRecursiveIter;
     // sorted variant of XIter
     class SortedXIter;
-    
+
+#ifndef SWIG
     // lists of iterators
     DeclareWvList(Iter);
+#endif  // SWIG
 };
 
 
+#ifndef SWIG
+
 /**
  * An implementation base class for key iterators.
  */
@@ -416,11 +472,16 @@ public:
         { it->rewind(); }
     bool next()
     {
-        if (! it->next())
+        if (!it->next())
             return false;
         current = top[it->key()];
         return true;
-    }   
+    }
+    
+    // FIXME: this is a speed optimization only.  Don't use this unless
+    // you're apenwarr.  It will change.
+    WvString _value() const
+        { return it->value(); }
 };
 
 
@@ -429,14 +490,29 @@ public:
  */
 class UniConf::RecursiveIter : public UniConf::IterBase
 {
-    UniConf::IterList itlist;
+    UniConfGen::Iter *it;
 
 public:
     /** Creates a recursive iterator over a branch. */
     RecursiveIter(const UniConf &_top);
 
-    void rewind();
-    bool next();
+    ~RecursiveIter()
+        { delete it; }
+
+    void rewind()
+        { it->rewind(); }
+    bool next()
+    {
+        if (!it->next())
+            return false;
+        current = top[it->key()];
+        return true;
+    }   
+
+    // FIXME: this is a speed optimization only.  Don't use this unless
+    // you're apenwarr.  It will change.
+    WvString _value() const
+        { return it->value(); }
 };
 
 
@@ -580,22 +656,23 @@ public:
         { populate(i); }
 };
 
-extern "C" {
+#endif  // SWIG
+
 #endif /* __cplusplus */
 
 
-#ifdef SWIG
-%module UniConf
+/**
+ * C Interface
+ */
 
-%{
-#include "uniconf.h"
-%}
 
-//%newobject uniconf_get;
-#endif
+#ifdef __cplusplus
+extern "C" {
+#endif  // __cplusplus
 
-/* FIXME: put the C binding here. */
-typedef void* uniconf_t;
+
+/** An opaque handle to a Uniconf object */
+typedef void *uniconf_t;
 
 
 /* Initialize and destroy UniConf. */
@@ -616,6 +693,7 @@ void uniconf_set(uniconf_t _uniconf,
 
 #ifdef	__cplusplus
 }
-#endif
+#endif  // __cplusplus
+
 
 #endif // __UNICONF_H
