@@ -10,6 +10,13 @@
 #include "wvfdstream.h"
 #include <fcntl.h>
 
+#ifdef _WIN32
+#include <io.h>
+#define O_NONBLOCK 0
+#define O_LARGEFILE 0
+#define fcntl(a,b,c)
+#endif
+
 /**
  * WvFile implements a stream connected to a file or Unix device.  We
  * include no support for operations like seek().  Since files are not
@@ -23,8 +30,7 @@ class WvFile : public WvFDStream
 {
 public:
     WvFile(int rwfd = -1);
-    WvFile(WvStringParm filename, int mode, int create_mode = 0666)
-        { open(filename, mode, create_mode); }
+    WvFile(WvStringParm filename, int mode, int create_mode = 0666);
     bool open(WvStringParm filename, int mode, int create_mode = 0666);
     
     bool readable, writable;
