@@ -10,7 +10,8 @@
 #include "wvstreamclone.h"
 #include "wvfdstream.h"
 #include "wvlog.h"
- 
+#include "wvcallback.h"
+
 struct ssl_st;
 struct ssl_ctx_st;
 struct ssl_method_st;
@@ -21,6 +22,8 @@ typedef struct ssl_method_st SSL_METHOD;
 
 class WvX509Mgr;
 
+typedef WvCallback<bool, WvX509Mgr*> WvSSLValidateCallback;
+
 /**
  * SSL Stream, handles SSLv2, SSLv3, and TLS
  * Methods - If you want it to be a server, then you must feed the constructor
@@ -31,10 +34,11 @@ class WvSSLStream : public WvStreamClone
 public:
     /**  
      * Start an SSL connection on the stream _slave.  The x509 structure
-     * is optional for a client, and mandatory for a server.
+     * is optional for a client, and mandatory for a server.  You need to
+     * keep the X509 object around for the entire life of this object!
      */
     WvSSLStream(IWvStream *_slave, WvX509Mgr *x509 = NULL, 
-    		bool _verify = false, bool _is_server = false);
+    		WvSSLValidateCallback _vcb = 0, bool _is_server = false);
     
     /** Cleans up everything (calls close + frees up the SSL Objects used) */
     virtual ~WvSSLStream();
@@ -88,7 +92,7 @@ private:
     bool is_server;
     
     /** Keep track of whether we want to check the peer who connects to us */
-    bool verify;
+    WvSSLValidateCallback vcb;
     
     /** Internal Log Object */
     WvLog debug;
@@ -113,6 +117,7 @@ private:
 
     /** Prints out the entire SSL error queue */
     void printerr(WvStringParm func);
+    
 };
 
 #endif // __WVSSLSTREAM_H
