@@ -7,10 +7,7 @@
 #ifndef __WVSTREAM_H
 #define __WVSTREAM_H
 
-#include "wvxplc.h"
-#include "wverror.h"
-#include "wvbuf.h"
-#include "wvcallback.h"
+#include "iwvstream.h"
 #include "wvtimeutils.h"
 #include <errno.h>
 #include <limits.h>
@@ -24,123 +21,6 @@
 #include <sys/time.h>
 #endif
 
-class WvAddr;
-class WvTask;
-class WvTaskMan;
-class WvStream;
-
-// parameters are: owning-stream, userdata
-typedef WvCallback<void, WvStream&, void*> WvStreamCallback;
-
-class IWvStream : public IObject, public WvError
-{
-public:
-    /**
-     * A SelectRequest is a convenient way to remember what we want to do
-     * to a particular stream: read from it, write to it, or check for
-     * exceptions.
-     */
-    struct SelectRequest {
-	bool readable, writable, isexception;
-	
-	SelectRequest() { }
-	SelectRequest(bool r, bool w, bool x = false)
-		{ readable = r; writable = w; isexception = x; }
-	
-	SelectRequest &operator |= (const SelectRequest &r)
-		{ readable |= r.readable; writable |= r.writable;
-		    isexception |= r.isexception; return *this; }
-    };
-    
-    /**
-     * the data structure used by pre_select()/post_select() and internally
-     * by select().
-     */
-    struct SelectInfo {
-	fd_set read, write, except; // set by pre_select, read by post_select
-	SelectRequest wants;        // what is the user looking for?
-	int max_fd;                 // largest fd in read, write, or except
-	time_t msec_timeout;        // max time to wait, or -1 for forever
-	bool inherit_request;       // 'wants' values passed to child streams
-	bool global_sure;           // should we run the globalstream callback
-    };
-    
-    IWvStream();
-    virtual ~IWvStream();
-    virtual void close() = 0;
-    virtual bool isok() const = 0;
-    virtual void callback() = 0;
-    
-    // FIXME: these really have no place in the interface...
-    virtual int getrfd() const = 0;
-    virtual int getwfd() const = 0;
-
-    // FIXME: evil, should go away (or be changed to localaddr/remoteaddr)
-    virtual const WvAddr *src() const = 0;
-    
-    // needed for select().
-    // Some say that pre_select() should go away.
-    virtual bool pre_select(SelectInfo &si) = 0;
-    virtual bool post_select(SelectInfo &si) = 0;
-    
-    // these are now the official way to get/put data to your stream.
-    // The old uread() and uwrite() are just implementation details!
-    virtual size_t read(void *buf, size_t count) = 0;
-    virtual size_t write(const void *buf, size_t count) = 0;
-
-    // FIXME: these are the new fancy versions, but WvBuf needs to have
-    // a safely abstracted interface class (IWvBuf) before IWvStream will
-    // really be safe, if we include these.
-    virtual size_t read(WvBuf &outbuf, size_t count) = 0;
-    virtual size_t write(WvBuf &inbuf, size_t count = INT_MAX) = 0;
-
-    /**
-     * Shuts down the reading side of the stream.
-     * Subsequent calls to read() will fail.
-     */
-    virtual void noread() = 0;
-
-    /**
-     * Shuts down the writing side of the stream.
-     * Subsequent calls to write() will fail.
-     */
-    virtual void nowrite() = 0;
-    
-    /** Returns true if the stream is readable. */
-    virtual bool isreadable() = 0;
-    
-    /** Returns true if the stream is writable (without using the outbuf). */
-    virtual bool iswritable() = 0;
-    
-    /**
-     * flush the output buffer, if we can do it without delaying more than
-     * msec_timeout milliseconds at a time.  (-1 means wait forever)
-     * 
-     * Returns true if it finished flushing (ie. the outbuf is empty).
-     * 
-     * FIXME: Something like this probably belongs in IWvStream, but
-     * probably not exactly this.
-     */
-    virtual bool flush(time_t msec_timeout) = 0;
-
-    /**
-     * Returns true if we want to flush the output buffer right now.  This
-     * allows us to implement delayed_flush(), flush_then_close(), etc, but
-     * it's still super-ugly and probably needs to go away.  (In fact, all
-     * our buffer flushing is super-ugly right now.)
-     */
-    virtual bool should_flush() = 0;
-
-    /** Sets a callback to be invoked on close().  */
-    virtual void setclosecallback(WvStreamCallback _callfunc,
-				  void *_userdata) = 0;
-    
-    // IObject
-    static const XUUID XIID;
-};
-
-DEFINE_XIID(IWvStream, {0x7ca76e98, 0xb653, 0x43d7,
-    {0xb0, 0x56, 0x8b, 0x9d, 0xde, 0x9a, 0xbe, 0x9d}});
 
 /**
  * Unified support for streams, that is, sequences of bytes that may or
@@ -149,8 +29,9 @@ DEFINE_XIID(IWvStream, {0x7ca76e98, 0xb653, 0x43d7,
  * We provide typical read and write routines, as well as a select() function
  * for each stream.
  */
-class WvStream : public GenericComponent<IWvStream>
+class WvStream: public IWvStream
 {
+    IMPLEMENT_IOBJECT(WvStream);
 public:
     /**
      * 'force' is the list of default SelectRequest values when you use the
@@ -182,6 +63,9 @@ public:
      */
     bool alarm_was_ticking;
     
+    /** True if noread()/nowrite()/close() have been called, respectively. */
+    bool stop_read, stop_write, closed;
+    
     /** Basic constructor for just a do-nothing WvStream */
     WvStream();
     virtual ~WvStream();
@@ -198,7 +82,7 @@ public:
     /** Override seterr() from WvError so that it auto-closes the stream. */
     virtual void seterr(int _errnum);
     void seterr(WvStringParm specialerr)
-        { WvError::seterr(specialerr); }
+        { WvErrorBase::seterr(specialerr); }
     void seterr(WVSTRING_FORMAT_DECL)
         { seterr(WvString(WVSTRING_FORMAT_CALL)); }
     
@@ -275,6 +159,7 @@ public:
 
     virtual void noread();
     virtual void nowrite();
+    virtual void maybe_autoclose();
     
     virtual bool isreadable();
     virtual bool iswritable();
@@ -297,7 +182,7 @@ public:
      * This is what you would override in a derived class.
      */ 
     virtual size_t uwrite(const void *buf, size_t count)
-        { return 0; /* basic WvStream doesn't actually do anything! */ }
+        { return count; /* basic WvStream doesn't actually do anything! */ }
     
     /**
      * read up to one line of data from the stream and return a pointer
@@ -322,6 +207,26 @@ public:
     char *getline(time_t wait_msec, char separator = '\n',
 		  int readahead = 1024);
     
+    /**
+     * read up to count characters into buf, up to and including the first
+     * instance of separator.
+     * 
+     * if separator is not found on input before timeout (usual symantics)
+     * or stream close or error, or if count is 0, nothing is placed in buf
+     * and 0 is returned.
+     * 
+     * if your buffer is not large enough for line, call multiple times
+     * until seperator is found at end of buffer to retrieve the entire
+     * line.
+     * 
+     * Returns the number of characters that were put in buf.
+     * 
+     * If uses_continue_select is true, getline() will use
+     * continue_select() rather than select() to wait for its timeout.
+     */
+    size_t read_until(void *buf, size_t count, time_t wait_msec,
+                      char separator);
+
     /**
      * force read() to not return any bytes unless 'count' bytes can be
      * read at once.  (Useful for processing Content-Length headers, etc.)
@@ -536,7 +441,7 @@ public:
     
     /**
      * return to the caller from execute(), but don't really return exactly;
-     * this uses WvTaskMan::yield() to return to the caller of callback()
+     * this uses WvCont::yield() to return to the caller of callback()
      * without losing our place in execute() itself.  So, next time someone
      * calls callback(), it will be as if continue_select() returned.
      * 
@@ -547,7 +452,7 @@ public:
      * 
      * NOTE 2: if you're going to call continue_select(), you should set
      * uses_continue_select=true before the first call to callback().
-     * Otherwise your WvTask struct won't get created.
+     * Otherwise your WvCont won't get created.
      * 
      * NOTE 3: if msec_timeout >= 0, this uses WvStream::alarm().
      */
@@ -570,12 +475,10 @@ public:
      * define the callback function for this stream, called whenever
      * the callback() member is run, and passed the 'userdata' pointer.
      */
-    void setcallback(WvStreamCallback _callfunc, void *_userdata)
-        { callfunc = _callfunc; userdata = _userdata; }
+    void setcallback(WvStreamCallback _callfunc, void *_userdata);
         
     /** Sets a callback to be invoked on close().  */
-    void setclosecallback(WvStreamCallback _callfunc, void *_userdata)
-       { closecb_func = _callfunc; closecb_data = _userdata; }
+    void setclosecallback(WvStreamCallback _callfunc, void *_userdata);
 
     /**
      * set the callback function for this stream to an internal routine
@@ -588,6 +491,16 @@ public:
     void noautoforward();
     static void autoforward_callback(WvStream &s, void *userdata);
     
+    /**
+     * A wrapper that's compatible with WvCont, but calls the "real" callback.
+     */
+    void *_callwrap(void *);
+    
+    /**
+     * Actually call the registered callfunc and execute().
+     */
+    void _callback();
+    
     /**
      * if the stream has a callback function defined, call it now.
      * otherwise call execute().
@@ -667,17 +580,14 @@ private:
 
 
 protected:
-    WvTaskMan *taskman;
-
     WvDynBuf inbuf, outbuf;
-    WvStreamCallback callfunc;
-    WvStreamCallback closecb_func;
+    WvStreamCallback callfunc, closecb_func;
+    WvCallback<void*,void*> call_ctx;
     void *userdata;
     void *closecb_data;
     size_t max_outbuf_size;
     bool outbuf_delayed_flush;
     bool is_auto_flush;
-    bool want_nowrite;
 
     // Used to guard against excessive flushing when using delay_flush
     bool want_to_flush;
@@ -689,22 +599,12 @@ protected:
     time_t autoclose_time;	// close eventually, even if output is queued
     WvTime alarm_time;          // select() returns true at this time
     WvTime last_alarm_check;    // last time we checked the alarm_remaining
-    bool running_callback;	// already in the callback() function
     bool wvstream_execute_called;
     
-    WvTask *task;
-
     /** Prevent accidental copying of WvStreams. */
     WvStream(const WvStream &s) { }
     WvStream& operator= (const WvStream &s) { return *this; }
 
-    /**
-     * actually do the callback for an arbitrary stream.
-     * This is a static function so we can pass it as a function pointer
-     * to WvTask functions.
-     */
-    static void _callback(void *stream);
-    
     /**
      * The callback() function calls execute(), and then calls the user-
      * specified callback if one is defined.  Do not call execute() directly;
