@@ -20,9 +20,15 @@
 #include <unistd.h>
 #include <errno.h>
 #include <linux/sockios.h>
+
+#define _LINUX_IF_H /* Hack to prevent loading linux/if.h */
 #include <linux/wireless.h>
 
-#define min(x,y) ((x) < (y) ? (x) : (y))
+#define min(x,y) ({ \
+    const typeof(x) _x = (x); \
+    const typeof(y) _y = (y); \
+    (void) (&_x == &_y); \
+    _x < _y ? _x : _y; })
 
 WvInterfaceDictBase WvInterfaceDict::slist(15);
 int WvInterfaceDict::links = 0;
@@ -51,7 +57,7 @@ int WvInterface::req(int ioctl_num, struct ifreq *ifr)
     ifr->ifr_name[IFNAMSIZ-1] = 0;
     
     retval = ioctl(sock, ioctl_num, ifr);
-    if (retval)
+    if (retval == -1)
         retval = errno;
     close(sock);
     return retval;
