@@ -5,6 +5,7 @@
  * Declarations for wvcon, wvin, wvout, and wverr global streams.
  */
 #include "wvfdstream.h"
+#include "wvmoniker.h"
 
 // just like WvFDStream, but doesn't close the fd
 class _WvConStream : public WvFDStream
@@ -19,6 +20,30 @@ public:
 };
 
 
+static IWvStream *create_stdin(WvStringParm s, IObject *, void *)
+{
+    return new _WvConStream(0, -1);
+}
+static IWvStream *create_stdout(WvStringParm s, IObject *, void *)
+{
+    return new _WvConStream(-1, 1);
+}
+static IWvStream *create_stderr(WvStringParm s, IObject *, void *)
+{
+    return new _WvConStream(-1, 2);
+}
+static IWvStream *create_stdio(WvStringParm s, IObject *, void *)
+{
+    return new _WvConStream(0, 1);
+}
+
+static WvMoniker<IWvStream> reg0("stdin",  create_stdin);
+static WvMoniker<IWvStream> reg1("stdout", create_stdout);
+static WvMoniker<IWvStream> reg2("stderr", create_stderr);
+static WvMoniker<IWvStream> reg3("stdio",  create_stdio);
+
+
+
 _WvConStream::_WvConStream(int _rfd, int _wfd) : WvFDStream(_rfd, _wfd)
 {
     isopen = true;
@@ -48,7 +73,7 @@ bool _WvConStream::isok() const
 // console streams
 #ifdef _WIN32
 
-#include "../Win32WvStreams/streams/streams.h"
+#include "streams.h"
 SocketFromFDMaker _zero(0, fd2socket_fwd);
 SocketFromFDMaker _one(1, socket2fd_fwd);
 SocketFromFDMaker _two(2, socket2fd_fwd);
