@@ -9,6 +9,8 @@
 #include "wvlogrcv.h"
 #include "wvstringlist.h"
 #include "strutils.h"
+#include "wvfork.h"
+
 #include <ctype.h>
 
 #ifdef _WIN32
@@ -125,6 +127,8 @@ size_t WvLog::uwrite(const void *_buf, size_t len)
 
 WvLogRcvBase::WvLogRcvBase()
 {
+    static_init();
+    WvLogRcvBase::force_new_line = false;
     WvLog::receivers.append(this, false);
     WvLog::num_receivers++;
 }
@@ -139,7 +143,34 @@ WvLogRcvBase::~WvLogRcvBase()
 
 const char *WvLogRcvBase::appname(const WvLog *log) const
 {
-    return log->app;
+    if (log)
+	return log->app;
+    else
+	return WvString("unknown");
+}
+
+
+void WvLogRcvBase::static_init()
+{
+    static bool init = false;
+    if (!init)
+    {
+#ifndef _WIN32
+        add_wvfork_callback(WvLogRcvBase::cleanup_on_fork);
+#endif
+        init = true;
+    }
+}
+
+
+void WvLogRcvBase::cleanup_on_fork(pid_t p)
+{
+    if (p) return;      // parent: do nothing
+
+    WvLog::receivers.zap();
+    delete WvLog::default_receiver;
+    WvLog::default_receiver = NULL;
+    WvLog::num_receivers = 0;
 }
 
 
@@ -222,7 +253,7 @@ void WvLogRcv::log(const WvLog *source, int _loglevel,
     // only need to start a new line with new headers if they headers have
     // changed.  if the source and level are the same as before, just continue
     // the previous log entry.
-    if (source != last_source || loglevel != last_level)
+    if (source != last_source || loglevel != last_level || WvLogRcvBase::force_new_line)
     {
 	end_line();
 	last_source = source;
@@ -266,21 +297,21 @@ void WvLogRcv::log(const WvLog *source, int _loglevel,
 		break;
 	}
 	
-	if (*cptr == '\n') // end of line
+	if (cptr >= bufend) // end of buffer
+	{
+	    mid_line(buf, bufend - buf);
+	    buf = bufend;
+	}
+	else if (*cptr == '\n') // end of line
 	{
 	    mid_line((const char *)buf, cptr - buf);
 	    buf = cptr;
 	}
-	else if (!my_isprint(*cptr))
+	else // therefore (!my_isprint(*cptr))
 	{
 	    mid_line(buf, cptr - buf);
 	    buf = cptr;
 	}
-	else // end of buffer
-	{
-	    mid_line(buf, bufend - buf);
-	    buf = bufend;
-	}
     }
 }
 
