@@ -4,18 +4,36 @@
  *
  * A "Log Receiver" that logs messages to a file
  */
-
-#include <time.h>
 #include "wvlogfile.h"
 #include "wvtimeutils.h"
 #include "wvdiriter.h"
 #include "strutils.h"
+#include "wvdailyevent.h"
+#include "wvfork.h"
+#include <time.h>
+#include <sys/types.h>
+#include <sys/wait.h>
 
 #define MAX_LOGFILE_SZ	1024*1024*100	// 100 Megs
 
 
 //----------------------------------- WvLogFileBase ------------------
 
+WvLogFileBase::WvLogFileBase(WvStringParm _filename, WvLog::LogLevel _max_level)
+    : WvLogRcv(_max_level),
+      WvFile(_filename, O_WRONLY|O_APPEND|O_CREAT|O_LARGEFILE, 0644)
+{
+    // nothing special
+}
+
+
+WvLogFileBase::WvLogFileBase(WvLog::LogLevel _max_level) 
+    : WvLogRcv(_max_level) 
+{ 
+    // nothing special
+}
+
+
 void WvLogFileBase::_mid_line(const char *str, size_t len)
 {
     WvFile::write(str, len);
@@ -36,9 +54,10 @@ void WvLogFileBase::_make_prefix()
 //----------------------------------- WvLogFile ----------------------
 
 WvLogFile::WvLogFile(WvStringParm _filename, WvLog::LogLevel _max_level,
-     int _keep_for) : WvLogFileBase(_max_level), keep_for(_keep_for),
-     filename(_filename)
+     int _keep_for, bool _force_new_line) : WvLogFileBase(_max_level), keep_for(_keep_for),
+					    filename(_filename)
 {
+    WvLogRcv::force_new_line = _force_new_line;
     start_log();
 }
 
@@ -52,9 +71,8 @@ void WvLogFile::_make_prefix()
     if (fstat(getfd(), &statbuf) == -1)
         statbuf.st_size = 0;
 
-    // Make sure we are calculating last_day in the current time zone.
-    if (last_day < ((timenow + tmstamp->tm_gmtoff)/86400) 
-	|| statbuf.st_size > MAX_LOGFILE_SZ)
+    // Start a new log when it's a new day or the file is too big
+    if (last_day != tmstamp->tm_yday || statbuf.st_size > MAX_LOGFILE_SZ)
         start_log();
 
     WvLogFileBase::_make_prefix();
@@ -69,7 +87,7 @@ void WvLogFile::start_log()
     struct stat statbuf;
     time_t timenow = wvtime().tv_sec;
     struct tm* tmstamp = localtime(&timenow);
-    last_day = (timenow + tmstamp->tm_gmtoff) / 86400;
+    last_day = tmstamp->tm_yday;
     char buf[20];
     WvString fullname;
     strftime(buf, 20, "%Y-%m-%d", tmstamp);
@@ -92,19 +110,32 @@ void WvLogFile::start_log()
         symlink(getfilename(fullname), curname);
     }
 
-    // Look for old logs and purge them
-    WvDirIter i(getdirname(filename), false);
-    i.rewind();
-    while (i.next() && keep_for)
+    // We fork here because this can be really slow when the directory has (oh, say 32,000 files)
+    pid_t forky = wvfork();
+    if (!forky)
     {
-        // if it begins with the base name
-        if (!strncmp(i.ptr()->name, base, strlen(base)))
-            // and it's older than 'keep_for' days
-            if (i.ptr()->st_mtime <
+	// In child
+	if (!wvfork())
+	{
+	    // Child will Look for old logs and purge them
+	    WvDirIter i(getdirname(filename), false);
+	    i.rewind();
+	    while (i.next() && keep_for)
+	    {
+		// if it begins with the base name
+		if (!strncmp(i.ptr()->name, base, strlen(base)))
+		    // and it's older than 'keep_for' days
+		    if (i.ptr()->st_mtime <
                     wvtime().tv_sec - keep_for*86400)
-            {
-                //delete it
-                unlink(i.ptr()->fullname);
-            }
+		    {
+			//delete it
+			unlink(i.ptr()->fullname);
+		    }
+	    }
+	    _exit(0);
+	}
+	_exit(0);
     }
+    int status;
+    waitpid(forky, &status, 0);
 }
