@@ -5,6 +5,7 @@
  * UniClientGen is a UniConfGen for retrieving data from the
  * UniConfDaemon.
  */
+#include "wvfile.h"
 #include "uniclientgen.h"
 #include "wvtclstring.h"
 #include "wvtcp.h"
@@ -13,16 +14,18 @@
 #include "wvmoniker.h"
 #include "wvsslstream.h"
 
+
 #ifndef _WIN32
 #include "wvunixsocket.h"
-static UniConfGen *unixcreator(WvStringParm s, IObject *, void *)
+static IUniConfGen *unixcreator(WvStringParm s, IObject *, void *)
 {
     return new UniClientGen(new WvUnixConn(s));
 }
-static WvMoniker<UniConfGen> unixreg("unix", unixcreator);
+static WvMoniker<IUniConfGen> unixreg("unix", unixcreator);
 #endif
 
-static UniConfGen *tcpcreator(WvStringParm _s, IObject *, void *)
+
+static IUniConfGen *tcpcreator(WvStringParm _s, IObject *, void *)
 {
     WvString s(_s);
     char *cptr = s.edit();
@@ -33,7 +36,8 @@ static UniConfGen *tcpcreator(WvStringParm _s, IObject *, void *)
     return new UniClientGen(new WvTCPConn(s), _s);
 }
 
-static UniConfGen *sslcreator(WvStringParm _s, IObject *, void *)
+
+static IUniConfGen *sslcreator(WvStringParm _s, IObject *, void *)
 {
     WvString s(_s);
     char *cptr = s.edit();
@@ -41,12 +45,13 @@ static UniConfGen *sslcreator(WvStringParm _s, IObject *, void *)
     if (!strchr(cptr, ':')) // no default port
 	s.append(":%s", DEFAULT_UNICONF_DAEMON_SSL_PORT);
     
-    return new UniClientGen(new WvSSLStream(new WvTCPConn(s), NULL, true), _s);
+    return new UniClientGen(new WvSSLStream(new WvTCPConn(s), NULL), _s);
 }
 
+
 // if 'obj' is a WvStream, build the uniconf connection around that;
 // otherwise, create a new WvStream using 's' as the wvstream moniker.
-static UniConfGen *wvstreamcreator(WvStringParm s, IObject *obj, void *)
+static IUniConfGen *wvstreamcreator(WvStringParm s, IObject *obj, void *)
 {
     IWvStream *stream = NULL;
     if (obj)
@@ -56,34 +61,54 @@ static UniConfGen *wvstreamcreator(WvStringParm s, IObject *obj, void *)
     return new UniClientGen(stream);
 }
 
-static WvMoniker<UniConfGen> tcpreg("tcp", tcpcreator);
-static WvMoniker<UniConfGen> sslreg("ssl", sslcreator);
-static WvMoniker<UniConfGen> wvstreamreg("wvstream", wvstreamcreator);
+#ifdef WITH_SLP
+#include "wvslp.h"
+
+// FIXME: Only gets the first
+static IUniConfGen *slpcreator(WvStringParm s, IObject *obj, void *)
+{
+    WvStringList serverlist;
+    
+    if (slp_get_servs("uniconf.niti", serverlist))
+    {
+	WvString server = serverlist.popstr();
+	printf("Creating connection to: %s\n", server.cstr());
+	return new UniClientGen(new WvTCPConn(server), s);
+    }
+    else
+        return NULL;
+}
+
+static WvMoniker<IUniConfGen> slpreg("slp", slpcreator);
+#endif
+
+static WvMoniker<IUniConfGen> tcpreg("tcp", tcpcreator);
+static WvMoniker<IUniConfGen> sslreg("ssl", sslcreator);
+static WvMoniker<IUniConfGen> wvstreamreg("wvstream", wvstreamcreator);
+
 
 
 
 /***** UniClientGen *****/
 
-UniClientGen::UniClientGen(IWvStream *stream, WvStringParm dst) :
-    conn(NULL), log(WvString("UniClientGen to %s",
-    dst.isnull() ? *stream->src() : WvString(dst))),
-    cmdinprogress(false), cmdsuccess(false)
+UniClientGen::UniClientGen(IWvStream *stream, WvStringParm dst) 
+    : log(WvString("UniClientGen to %s",
+		   dst.isnull() && stream->src() 
+		   ? *stream->src() : WvString(dst)))
 {
+    cmdinprogress = cmdsuccess = false;
+    result_list = NULL;
+
     conn = new UniClientConn(stream, dst);
     conn->setcallback(WvStreamCallback(this,
         &UniClientGen::conncallback), NULL);
-
-    deltastream.setcallback(WvStreamCallback(this, &UniClientGen::deltacb), 0);
-    WvIStreamList::globallist.append(&deltastream, false);
 }
 
 
 UniClientGen::~UniClientGen()
 {
-    WvIStreamList::globallist.unlink(&deltastream);
-
     conn->writecmd(UniClientConn::REQ_QUIT, "");
-    delete conn;
+    WVRELEASE(conn);
 }
 
 
@@ -99,6 +124,18 @@ bool UniClientGen::refresh()
     return true;
 }
 
+void UniClientGen::flush_buffers()
+{
+    // this ensures that all keys pending notifications are dealt with
+    while (conn->isok() && conn->isreadable())
+        conn->callback();
+}
+
+void UniClientGen::commit()
+{
+   UniConfKey tempkey("dummykey");
+   get(tempkey); // NOOP command, to ensure that all requests are flushed
+}
 
 WvString UniClientGen::get(const UniConfKey &key)
 {
@@ -109,8 +146,8 @@ WvString UniClientGen::get(const UniConfKey &key)
     {
         if (result_key == key)
             value = result;
-        else
-            seterror("Error: server sent wrong key pair.");
+//        else
+//            seterror("Error: server sent wrong key pair.");
     }
     return value;
 }
@@ -145,16 +182,46 @@ bool UniClientGen::haschildren(const UniConfKey &key)
 }
 
 
-UniClientGen::Iter *UniClientGen::iterator(const UniConfKey &key)
+UniClientGen::Iter *UniClientGen::do_iterator(const UniConfKey &key,
+					      bool recursive)
 {
-    result_list = new WvStringList();
-    conn->writecmd(UniClientConn::REQ_SUBTREE, wvtcl_escape(key));
+    assert(!result_list);
+    result_list = new KeyValList;
+    conn->writecmd(UniClientConn::REQ_SUBTREE,
+		   WvString("%s %s", wvtcl_escape(key), WvString(recursive)));
 
     if (do_select())
-        return new RemoteKeyIter(result_list);
+    {
+	ListIter *it = new ListIter(this);
+	KeyValList::Iter i(*result_list);
+	for (i.rewind(); i.next(); )
+	{
+	    it->keys.append(new WvString(i->key), true);
+	    it->values.append(new WvString(i->val), true);
+	}
+	
+	delete result_list;
+	result_list = NULL;
+        return it;
+    }
+    else
+    {
+	delete result_list;
+	result_list = NULL;
+	return NULL;
+    }
+}
 
-    delete result_list;
-    return new UniConfGen::NullIter();
+
+UniClientGen::Iter *UniClientGen::iterator(const UniConfKey &key)
+{
+    return do_iterator(key, false);
+}
+    
+
+UniClientGen::Iter *UniClientGen::recursiveiterator(const UniConfKey &key)
+{
+    return do_iterator(key, true);
 }
     
 
@@ -163,7 +230,7 @@ void UniClientGen::conncallback(WvStream &stream, void *userdata)
     if (conn->alarm_was_ticking)
     {
         // command response took too long!
-        log(WvLog::Error, "Command timeout; connection closed.\n");
+        log(WvLog::Warning, "Command timeout; connection closed.\n");
         cmdinprogress = false;
         cmdsuccess = false;
         conn->close();
@@ -172,7 +239,6 @@ void UniClientGen::conncallback(WvStream &stream, void *userdata)
     }
 
     UniClientConn::Command command = conn->readcmd();
-
     switch (command)
     {
         case UniClientConn::NONE:
@@ -230,7 +296,7 @@ void UniClientGen::conncallback(WvStream &stream, void *userdata)
                 if (!key.isnull() && !value.isnull())
                 {
                     if (result_list)
-                        result_list->append(new WvString(key), true);
+                        result_list->append(new KeyVal(key, value), true);
                 }
                 break;
             }
@@ -255,7 +321,7 @@ void UniClientGen::conncallback(WvStream &stream, void *userdata)
             {
                 WvString key(wvtcl_getword(conn->payloadbuf, " "));
                 WvString value(wvtcl_getword(conn->payloadbuf, " "));
-                clientdelta(key, value);
+                delta(key, value);
             }   
 
         default:
@@ -265,15 +331,22 @@ void UniClientGen::conncallback(WvStream &stream, void *userdata)
 }
 
 
+// FIXME: horribly horribly evil!!
 bool UniClientGen::do_select()
 {
+    hold_delta();
+    
     cmdinprogress = true;
     cmdsuccess = false;
 
     conn->alarm(TIMEOUT);
     while (conn->isok() && cmdinprogress)
     {
-        if (conn->select(-1))
+	// We would really like to run the "real" wvstreams globallist
+	// select loop here, but we can't because we may already be inside
+	// someone else's callback or something.  So we'll wait on *only* this
+	// connection.
+        if (conn->select(-1, true, false))
         {
             conn->callback();
             conn->alarm(TIMEOUT);
@@ -281,47 +354,10 @@ bool UniClientGen::do_select()
     }
     conn->alarm(-1);
 
-    if (!cmdsuccess)
-        seterror("Error: server timed out on response.");
+//    if (!cmdsuccess)
+//        seterror("Error: server timed out on response.");
 
+    unhold_delta();
+    
     return cmdsuccess;
 }
-
-
-
-/***** UniClientGen::RemoteKeyIter *****/
-
-void UniClientGen::RemoteKeyIter::rewind()
-{
-    i.rewind();
-}
-
-
-bool UniClientGen::RemoteKeyIter::next()
-{
-    return i.next();
-}
-
-
-UniConfKey UniClientGen::RemoteKeyIter::key() const
-{
-    return UniConfKey(*i).last();
-}
-
-void UniClientGen::clientdelta(const UniConfKey &key, WvStringParm value)
-{
-    deltas.append(new UniConfPair(key, value), true);
-    deltastream.alarm(0);
-}
-
-void UniClientGen::deltacb(WvStream &, void *)
-{
-    hold_delta();
-    UniConfPairList::Iter i(deltas);
-
-    for (i.rewind(); i.next(); )
-        delta(i->key(), i->value());
-
-    deltas.zap();
-    unhold_delta();
-}
