@@ -6,7 +6,7 @@
  */
 #include "uniconf.h"
 #include "uniconfroot.h"
-
+#include <assert.h>
 
 uniconf_t uniconf_init(const char* _moniker)
 {
@@ -26,7 +26,7 @@ const char* uniconf_get(uniconf_t _uniconf, const char* _key)
 {
     UniConfRoot* uniconf = static_cast<UniConfRoot*>(_uniconf);
 
-    return strdup((*uniconf)[WvString(_key)].get());
+    return strdup((*uniconf)[WvString(_key)].getme());
 }
 
 
@@ -35,6 +35,6 @@ void uniconf_set(uniconf_t _uniconf,
 {
     UniConfRoot* uniconf = static_cast<UniConfRoot*>(_uniconf);
 
-    return (*uniconf)[_key].set(_value);
+    return (*uniconf)[_key].setme(_value);
 }
 
