@@ -9,16 +9,16 @@
 
 // if 'obj' is non-NULL and is a UniConfGen, wrap that; otherwise wrap the
 // given moniker.
-static UniConfGen *creator(WvStringParm s, IObject *obj, void *)
+static IUniConfGen *creator(WvStringParm s, IObject *obj, void *)
 {
-    UniConfGen *gen = NULL;
+    IUniConfGen *gen = NULL;
     
     if (obj)
-	gen = mutate<UniConfGen>(obj);
+	gen = mutate<IUniConfGen>(obj);
     if (!gen)
-	gen = wvcreate<UniConfGen>(s);
+	gen = wvcreate<IUniConfGen>(s);
     
     return new UniReadOnlyGen(gen);
 }
 
-static WvMoniker<UniConfGen> reg("readonly", creator);
+static WvMoniker<IUniConfGen> reg("readonly", creator);
