@@ -7,7 +7,7 @@
 
 #include "wvbackslash.h"
 #include "wvstream.h"
-#include "wvstreamlist.h"
+#include "wvistreamlist.h"
 #include "wvencoderstream.h"
 #include <assert.h>
 
@@ -29,7 +29,7 @@ int main(int argc, char **argv)
     stream->auto_flush(false);
     stream->writechain.append(enc, true);
 
-    WvStreamList *slist = new WvStreamList();
+    WvIStreamList *slist = new WvIStreamList();
     slist->append(stream, false);
     slist->append(wvin, false);
     wvin->autoforward(*stream);
@@ -40,8 +40,8 @@ int main(int argc, char **argv)
             slist->callback();
     }
     stream->flush(0);
-    delete stream;
-    delete slist;
+    WVRELEASE(stream);
+    WVRELEASE(slist);
     
     return 0;
 }
