@@ -2,9 +2,13 @@
  * Do some repetitive stuff that *shouldn't* stress bdbhash very much, so
  * that somebody can strace it and see if it does.
  */
-#include "wvbdbhash.h"
+#include "wvautoconf.h"
 
-typedef WvBdbHash<int, WvString> MyHash;
+#if defined(WITH_BDB)
+#include "wvondiskhash.h"
+#include "wvstring.h"
+
+typedef WvOnDiskHash<int, WvString, WvBdbHash> MyHash;
 
 int main()
 {
@@ -41,3 +45,9 @@ int main()
     }
 #endif
 }
+#else
+int main(int argc, char *argv[])
+{
+  return 0;
+}
+#endif
