@@ -11,15 +11,17 @@
 #include "windows.h"
 #endif
 
-WvError::~WvError()
+WvErrorBase::~WvErrorBase()
 {
     // nothing special
 }
 
 
-WvString WvError::errstr() const
+WvString WvErrorBase::errstr() const
 {
-    if (errnum == -1)
+    int errnum = geterr();
+    
+    if (errnum < 0)
     {
 	assert(!!errstring);
 	return errstring;
@@ -44,15 +46,20 @@ WvString WvError::errstr() const
 }
 
 
-void WvError::seterr(int _errnum)
+void WvErrorBase::seterr(int _errnum)
 {
     if (!errnum)
+    {
+        assert(_errnum != -1 || !!errstring
+	    && "attempt to set errnum to -1 without also setting errstring");
 	errnum = _errnum;
+    }
 }
 
 
-void WvError::seterr(WvStringParm specialerr)
+void WvErrorBase::seterr(WvStringParm specialerr)
 {
+    assert(!!specialerr);
     if (!errnum)
     {
 	errstring = specialerr;
@@ -61,7 +68,7 @@ void WvError::seterr(WvStringParm specialerr)
 }
 
 
-void WvError::seterr(const WvError &err)
+void WvErrorBase::seterr(const WvErrorBase &err)
 {
     if (err.geterr() > 0)
 	seterr(err.geterr());
