@@ -126,14 +126,22 @@ WvFastString::WvFastString(unsigned long i)
 WvFastString::WvFastString(long long i)
 {
     newbuf(32);
+#ifdef _WIN32
+    sprintf(str, "%I64d", i);
+#else
     sprintf(str, "%lld", i);
+#endif
 }
 
 
 WvFastString::WvFastString(unsigned long long i)
 {
     newbuf(32);
+#ifdef _WIN32
+    sprintf(str, "%I64u", i);
+#else
     sprintf(str, "%llu", i);
+#endif
 }
 
 
@@ -247,10 +255,25 @@ WvString &WvString::operator= (int i)
 
 WvString &WvString::operator= (const WvFastString &s2)
 {
-    if (s2.buf == buf && s2.str == str)
+    if (s2.str == str && (!s2.buf || s2.buf == buf))
 	return *this; // no change
     else if (!s2.buf)
     {
+	// We have a string, and we're about to free() it.
+	if (str && buf && buf->links == 1)
+	{
+	    // Set buf->size, if we don't already know it.
+	    if (buf->size == 0)
+		buf->size = strlen(str);
+
+	    if (str < s2.str && s2.str <= (str + buf->size))
+	    {
+		// If the two strings overlap, we'll just need to
+		// shift s2.str over to here.
+		memmove(buf->data, s2.str, buf->size);
+		return *this;
+	    }
+	}
 	// assigning from a non-copied string - copy data if needed.
 	unlink();
 	link(&nullbuf, s2.str);
@@ -316,14 +339,15 @@ bool WvFastString::operator! () const
 }
 
 
-// parse a 'percent' operator from a format string.  For example:
-//        cptr      out:  zeropad  justify   maxlen  return pointer
-//        "%s"             false      0         0    "s"
-//        "%-15s"          false    -15         0    "s"
-//        "%15.5s"         false     15         5    "s"
-//        "%015.5s"        true      15         5    "s"
-// and so on.  On entry, cptr should _always_ point at a percent '%' char.
-//
+/** 
+ * parse a 'percent' operator from a format string.  For example:
+ *        cptr      out:  zeropad  justify   maxlen  return pointer
+ *        "%s"             false      0         0    "s"
+ *        "%-15s"          false    -15         0    "s"
+ *        "%15.5s"         false     15         5    "s"
+ *        "%015.5s"        true      15         5    "s"
+ * and so on.  On entry, cptr should _always_ point at a percent '%' char.
+ */
 static const char *pparse(const char *cptr,
 			  bool &zeropad, int &justify, int &maxlen)
 {
@@ -350,18 +374,19 @@ static const char *pparse(const char *cptr,
 }
 
 
-// Accept a printf-like format specifier (but more limited) and an array
-// of WvStrings, and render them into another WvString.  For example:
-//          WvString x[] = {"foo", "blue", 1234};
-//          WvString ret = WvString::do_format("%s%10.2s%-10s", x);
-//
-// The 'ret' string will be:  "foo        bl1234      "
-// Note that only '%s' is supported, though integers can be rendered
-// automatically into WvStrings.  %d, %f, etc are not allowed!
-//
-// This function is usually called from some other function which allocates
-// the array automatically.
-//
+/**
+ * Accept a printf-like format specifier (but more limited) and an array
+ * of WvStrings, and render them into another WvString.  For example:
+ *          WvString x[] = {"foo", "blue", 1234};
+ *          WvString ret = WvString::do_format("%s%10.2s%-10s", x);
+ *
+ * The 'ret' string will be:  "foo        bl1234      "
+ * Note that only '%s' is supported, though integers can be rendered
+ * automatically into WvStrings.  %d, %f, etc are not allowed!
+ *
+ * This function is usually called from some other function which allocates
+ * the array automatically.
+ */
 void WvFastString::do_format(WvFastString &output, const char *format,
 			     const WvFastString * const *a)
 {
